﻿/**
*@FILENAME		: DashBoard.cs
*@DESCRIPTION	: The DashBoard application displays the bin levels for each stations
*@VERSION		: April 27, 2017
*@PROGRAMMER	: Brian Hinds and Laura Fadrique
*/
using System;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Windows.Forms;

namespace BinDashBoard
{
    public partial class DashBorad : Form
    {
        /// <summary>
        /// Constant Integer variables used in this application
        /// </summary>
        private const int BinMin = 1;
        private const int BinLow = 25;
        private const int BinMid = 50;
        private const int BinHigh = 75;
        private const int BinPercentage = 100;
        /// <summary>
        ///  Integer variables used in this application
        /// </summary>
        private int HarnessMax = 0;
        private int ReflectorMax = 0;
        private int HousingMax = 0;
        private int LensMax = 0;
        private int BulbMax = 0;
        private int BezelMax = 0;
        private int HarnessValue = 0;
        private int ReflectorValue = 0;
        private int HousingValue = 0;
        private int LensValue = 0;
        private int BulbValue = 0;
        private int BezelValue = 0;

        private int StationId = 0;
        private double BinResult = 0;
        /// <summary>
        /// String variables used in this application
        /// </summary>
        private string StationDate = "";
        private string DashBoardConnectionString = "";
        private string StationID = "";

        private DateTime StationPDate;
        /// <summary>
        /// Instantiating classes that are used in the application
        /// </summary>
        private DashBoardConnection DBCON = new DashBoardConnection();
        private DashBoardCommands DBCMD = new DashBoardCommands();
       
        private bool isRunning = false;

        /// <summary>
        /// Constructor for application
        /// </summary>
        public DashBorad()
        {
            InitializeComponent();
            startBtn.Enabled = false;
            stopBtn.Enabled = false;
            KanbanStationBox.Enabled = false;
            RefreshRate.Enabled = false;
            stationUpdate.Enabled = false;
        }

        /// <summary>
        /// This method loads the connection string
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DashBorad_Load(object sender, EventArgs e)
        {
            DashBoardConnectionString = DBCON.KanbanConnection();
        }

        /// <summary>
        /// This method connects the dashboard
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectBtn_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(DashBoardConnectionString))
            {
                LoadStationIds();
                GetPartConfig();
                DashBoardIntializer();
            }
        }

        /// <summary>
        /// This method starts the dashboard
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startBtn_Click(object sender, EventArgs e)
        {
            connectBtn.Enabled = false;
            startBtn.Enabled = false;
            stopBtn.Enabled = true;

            DashboardRefresh.Interval = Convert.ToInt32(RefreshRate.Value);
            DashboardRefresh.Start();
        }

        /// <summary>
        /// This method stops the dashboard
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopBtn_Click(object sender, EventArgs e)
        {
            DashboardRefresh.Stop();
            startBtn.Enabled = true;
            stopBtn.Enabled = false;
            runningPIC.Image = Properties.Resources.standby;
            isRunning = false;
            LoadStationIds();
        }

        /// <summary>
        /// This method update pull the updates from the database to the dashboard sensors
        /// at a define interval
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DashboardRefresh_Tick(object sender, EventArgs e)
        {
            StationSelection();
            DashBoardDataStatus();
            DashBoardData();
            
        }

        /// <summary>
        /// This method initialize the dashboard sensors
        /// </summary>
        private void DashBoardIntializer()
        {
            // Display the ProgressBar control.
            HarnestPro.Visible = true;
            ReflectorPro.Visible = true;
            HousingPro.Visible = true;
            LensPro.Visible = true;
            BulbPro.Visible = true;
            BezelPro.Visible = true;
            // Set Minimum to 1
            HarnestPro.Minimum = BinMin;
            ReflectorPro.Minimum = BinMin;
            HousingPro.Minimum = BinMin;
            LensPro.Minimum = BinMin;
            BulbPro.Minimum = BinMin;
            BezelPro.Minimum = BinMin;
            // Set Maximum to the total number
            HarnestPro.Maximum = HarnessMax;
            ReflectorPro.Maximum = ReflectorMax;
            HousingPro.Maximum = HousingMax;
            LensPro.Maximum = LensMax;
            BulbPro.Maximum = BulbMax;
            BezelPro.Maximum = BezelMax;
        }

        /// <summary>
        /// This method updates the dashboard sensors
        /// </summary>
        private void DashBoardData()
        {
            if(HarnessValue >= BinMin) { HarnestPro.Value = HarnessValue; }

            if(ReflectorValue >= BinMin) { ReflectorPro.Value = ReflectorValue; }
           
            if(HousingValue >= BinMin) { HousingPro.Value = HousingValue; }

            if (LensValue >= BinMin) { LensPro.Value = LensValue; }
            
            if (BulbValue >= BinMin) { BulbPro.Value = BulbValue; }

           if (BezelValue >= BinMin) { BezelPro.Value = BezelValue; }
 
                      
        }

        /// <summary>
        /// This method adjust the interval the data is refreshed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RefreshRate_Scroll(object sender, EventArgs e)
        {
            DashboardRefresh.Interval = Convert.ToInt32(RefreshRate.Value);
        }

        /// <summary>
        /// This method adjust the bin status color by percentage
        /// </summary>
        private void DashBoardDataStatus()
        {
            /// Calculating Harness been percentage ///
            BinResult = (int)Math.Round(((double)HarnessValue / HarnessMax) * BinPercentage);

            hPercent.Text = BinResult.ToString() + " %";
            HP.Text = "(" + HarnessValue + "/" + HarnessMax + ")";

            if (BinResult > BinHigh)
            {
                HarnestPro.ForeColor = Color.Green;
            }
            if (BinResult < BinHigh)
            {
                HarnestPro.ForeColor = Color.YellowGreen;
            }
            if (BinResult < BinMid)
            {
                HarnestPro.ForeColor = Color.DarkOrange;
            }
            if (BinResult < BinLow)
            {
                HarnestPro.ForeColor = Color.Red;
            }

            /// Calculating Reflector been percentage ///
            BinResult = (int)Math.Round(((double)ReflectorValue / ReflectorMax) * BinPercentage);

            rPercent.Text = BinResult.ToString() + " %";
            RP.Text = "(" + ReflectorValue + "/" + ReflectorMax + ")";

            if (BinResult > BinHigh)
            {
                ReflectorPro.ForeColor = Color.Green;
            }
            if (BinResult < BinHigh)
            {
                ReflectorPro.ForeColor = Color.YellowGreen;
            }
            if (BinResult < BinMid)
            {
                ReflectorPro.ForeColor = Color.DarkOrange;
            }
            if (BinResult < BinLow)
            {
                ReflectorPro.ForeColor = Color.Red;
            }

            /// Calculating Housing been percentage ///
            BinResult = (int)Math.Round(((double)HousingValue / HousingMax) * BinPercentage);

            hoPercent.Text = BinResult.ToString() + " %";
            HP2.Text = "(" + HousingValue + "/" + HousingMax + ")";

            if (BinResult > BinHigh)
            {
                HousingPro.ForeColor = Color.Green;
            }
            if (BinResult < BinHigh)
            {
                HousingPro.ForeColor = Color.YellowGreen;
            }
            if (BinResult < BinMid)
            {
                HousingPro.ForeColor = Color.DarkOrange;
            }
            if (BinResult < BinLow)
            {
                HousingPro.ForeColor = Color.Red;
            }

            /// Calculating Lens been percentage ///
            BinResult = (int)Math.Round(((double)LensValue / LensMax) * BinPercentage);

            lPercent.Text = BinResult.ToString() + " %";
            LP.Text = "(" + LensValue + "/" + LensMax + ")";

            if (BinResult > BinHigh)
            {
                LensPro.ForeColor = Color.Green;
            }
            if (BinResult < BinHigh)
            {
                LensPro.ForeColor = Color.YellowGreen;
            }
            if (BinResult < BinMid)
            {
                LensPro.ForeColor = Color.DarkOrange;
            }
            if (BinResult < BinLow)
            {
                LensPro.ForeColor = Color.Red;
            }

            /// Calculating Bulb been percentage ///
            BinResult = (int)Math.Round(((double)BulbValue / BulbMax) * BinPercentage);

            buPercent.Text = BinResult.ToString() + " %";
            BP.Text = "(" + BulbValue + "/" + BulbMax + ")";

            if (BinResult > BinHigh)
            {
                BulbPro.ForeColor = Color.Green;
            }
            if (BinResult < BinHigh)
            {
                BulbPro.ForeColor = Color.YellowGreen;
            }
            if (BinResult < BinMid)
            {
                BulbPro.ForeColor = Color.DarkOrange;
            }
            if (BinResult < BinLow)
            {
                BulbPro.ForeColor = Color.Red;
            }

            /// Calculating Bezel been percentage ///
            BinResult = (int)Math.Round(((double)BezelValue / BezelMax) * BinPercentage);

            bePercent.Text = BinResult.ToString() + " %";
            BP2.Text = "(" + BezelValue + "/" + BezelMax + ")";

            if (BinResult > BinHigh)
            {
                BezelPro.ForeColor = Color.Green;
            }
            if (BinResult < BinHigh)
            {
                BezelPro.ForeColor = Color.YellowGreen;
            }
            if (BinResult < BinMid)
            {
                BezelPro.ForeColor = Color.DarkOrange;
            }
            if (BinResult < BinLow)
            {
                BezelPro.ForeColor = Color.Red;
            }
        }

        /// <summary>
        /// This method loads the station ids and production date
        /// </summary>
        private void LoadStationIds()
        {
            try
            {
                KanbanStationBox.DataSource = BinCommand(DBCMD.GetBinID());
                KanbanStationBox.DisplayMember = "station_id";
                KanbanStationBox.ValueMember = "date_production";
                connectionPIC.Image = Properties.Resources.ok;
                KanbanStationBox.Enabled = true;
                stationUpdate.Enabled = true;
            }
            catch
            {
                connectionPIC.Image = Properties.Resources.error;
                MessageBox.Show("Unable to retrieve station data.\nPlease check database connection config file\nThe incorrect database may have been chosen.", "Database Read Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();
            }
        }

        /// <summary>
        /// This method process bin commands
        /// </summary>
        /// <param name="CommandString"></param>
        /// <returns></returns>
        private DataTable BinCommand(string CommandString)
        {
            OleDbConnection con = new OleDbConnection(DashBoardConnectionString);
            con.Open();
            OleDbCommand cmd = con.CreateCommand();
            cmd.CommandType = CommandType.Text;
            cmd.CommandText = CommandString;
            cmd.ExecuteNonQuery();
            DataTable dt = new DataTable();
            OleDbDataAdapter da = new OleDbDataAdapter(cmd);
            da.Fill(dt);
            con.Close();
            return dt;
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void KanbanStationBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            StationDate = KanbanStationBox.SelectedValue.ToString();
            StationId = KanbanStationBox.SelectedIndex;
        }

        /// <summary>
        ///
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stationUpdate_Click(object sender, EventArgs e)
        {
            try
            {
                StationID = Convert.ToString(StationId + 1);
                StationPDate = Convert.ToDateTime(StationDate);
                if (!isRunning)
                {
                    startBtn.Enabled = true;
                    RefreshRate.Enabled = true;
                }

                runningPIC.Image = Properties.Resources.waiting;
            }
            catch
            {
                MessageBox.Show("Please select a station number", "Select A Station Number", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }

        }

        /// <summary>
        /// This method send the request to the database for bin data
        /// </summary>
        private void StationSelection()
        {
            try
            {
                runningPIC.Image = Properties.Resources.waiting;
                OleDbConnection con = new OleDbConnection(DashBoardConnectionString);

                OleDbCommand cmd = new OleDbCommand(DBCMD.GetBinData(), con);
                cmd.CommandType = CommandType.StoredProcedure;

                //Input
                OleDbParameter StationId = new OleDbParameter("@paramStationId", OleDbType.Integer);
                StationId.Value = Convert.ToInt32(StationID);
                StationId.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(StationId);

                OleDbParameter ProductionDate = new OleDbParameter("@paramDateProduction", OleDbType.Date);
                ProductionDate.Value = StationPDate;
                ProductionDate.Direction = ParameterDirection.Input;
                cmd.Parameters.Add(ProductionDate);

                ///Return Value
                OleDbParameter HarnessOut = new OleDbParameter("@paramHarnessOut", OleDbType.Integer);
                HarnessOut.Direction = ParameterDirection.ReturnValue;
                cmd.Parameters.Add(HarnessOut);

                OleDbParameter ReflectorOut = new OleDbParameter("@paramReflectorOut", OleDbType.Integer);
                ReflectorOut.Direction = ParameterDirection.ReturnValue;
                cmd.Parameters.Add(ReflectorOut);

                OleDbParameter HousingOut = new OleDbParameter("@paramHousingOut", OleDbType.Integer);
                HousingOut.Direction = ParameterDirection.ReturnValue;
                cmd.Parameters.Add(HousingOut);

                OleDbParameter LensOut = new OleDbParameter("@paramLensOut", OleDbType.Integer);
                LensOut.Direction = ParameterDirection.ReturnValue;
                cmd.Parameters.Add(LensOut);

                OleDbParameter BulbOut = new OleDbParameter("@paramBulbOut", OleDbType.Integer);
                BulbOut.Direction = ParameterDirection.ReturnValue;
                cmd.Parameters.Add(BulbOut);

                OleDbParameter BezelOut = new OleDbParameter("@paramBezelOut", OleDbType.Integer);
                BezelOut.Direction = ParameterDirection.ReturnValue;
                cmd.Parameters.Add(BezelOut);

                con.Open();

                ///Executing commands
                cmd.ExecuteNonQuery();

                con.Close();

                HarnessValue = Convert.ToInt32(HarnessOut.Value);
                ReflectorValue = Convert.ToInt32(ReflectorOut.Value);
                HousingValue = Convert.ToInt32(HousingOut.Value);
                LensValue = Convert.ToInt32(LensOut.Value);
                BulbValue = Convert.ToInt32(BulbOut.Value);
                BezelValue = Convert.ToInt32(BezelOut.Value);
                isRunning = true;
               runningPIC.Image = Properties.Resources.ok;
            }
            catch 
            {
                
                MessageBox.Show("Error retrieving current station bin data", "Bin Data Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                DashboardRefresh.Stop();
                startBtn.Enabled = true;
                stopBtn.Enabled = false;
                isRunning = false;
                runningPIC.Image = Properties.Resources.error;
            }
        }


        /// <summary>
        /// Get the minimum and maximum for each part
        /// </summary>
        private void GetPartConfig()
        {
            try
            {
                OleDbConnection con = new OleDbConnection(DashBoardConnectionString);
                con.Open();
                OleDbCommand cmd = con.CreateCommand();
                cmd.CommandType = CommandType.Text;
                cmd.CommandText = DBCMD.GetPartConfig();
                cmd.ExecuteNonQuery();
                DataTable dt = new DataTable();
                OleDbDataAdapter da = new OleDbDataAdapter(cmd);
                da.Fill(dt);

                foreach (DataRow row in dt.Rows)
                {
                    switch (Convert.ToInt32(row["part_id"]))
                    {
                        case 1:
                            HarnessMax = Convert.ToInt32(row["max_bin_capacity"]);
                            HarnessValue = Convert.ToInt32(row["min_bin_capacity"]);
                            HarnessMax += HarnessValue;
                            break;
                        case 2:
                            ReflectorMax = Convert.ToInt32(row["max_bin_capacity"]);
                            ReflectorValue = Convert.ToInt32(row["min_bin_capacity"]);
                            ReflectorMax += ReflectorValue;
                            break;
                        case 3:
                            HousingMax = Convert.ToInt32(row["max_bin_capacity"]);
                            HousingValue = Convert.ToInt32(row["min_bin_capacity"]);
                            HousingMax += HousingValue;
                            break;
                        case 4:
                            LensMax = Convert.ToInt32(row["max_bin_capacity"]);
                            LensValue = Convert.ToInt32(row["min_bin_capacity"]);
                            LensMax += LensValue;
                            break;
                        case 5:
                            BulbMax = Convert.ToInt32(row["max_bin_capacity"]);
                            BulbValue = Convert.ToInt32(row["min_bin_capacity"]);
                            BulbMax += BulbValue;
                            break;
                        case 6:
                            BezelMax = Convert.ToInt32(row["max_bin_capacity"]);
                            BezelValue = Convert.ToInt32(row["min_bin_capacity"]);
                            BezelMax += BezelValue;
                            break;
                    }
                }

                con.Close();
                
    }
            catch
            {
                connectionPIC.Image = Properties.Resources.error;
                MessageBox.Show("Unable to retrieve parts data.\nPlease check database connection settings\nThe incorrect database may have been chosen.", "Database Read Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                DashBoardConnectionString = DBCON.KanbanConnection();
            }
        }
    }
}