﻿/**
*@FILENAME		: KanbanStaion.cs
*@DESCRIPTION	: DashBoard application that monitors all station.
*@VERSION		: April 27, 2017
*@PROGRAMMER	: Brian Hinds and Laura Fadrique
*/
using System;
using System.Configuration;
using System.Data;
using System.Data.OleDb;
using System.Drawing;
using System.Windows.Forms;

namespace KanbanStationsDashBorad
{
    public partial class DashBoard : Form
    {
        private string DashBoardConnectionString = ConfigurationManager.ConnectionStrings["conn"].ConnectionString;

        private DashBoardCommads DBC = new DashBoardCommads();

        /// <summary>
        /// Variables for application
        /// </summary>
        private const int qMin = 1;
        private const int qLow = 25;
        private const int qMid = 50;
        private const int qHigh = 75;
        private int qResult;
        private int JouniorPass;
        private int JouniorFail;
        private int SeniorPass;
        private int SeniorFail;
        private int SuperPass;
        private int SuperFail;
        private bool status;
        private int JouniorMax;
        private int SeniorMax;
        private int SuperMax;
        private int JouniorQuota;
        private int SeniorQuota;
        private int SuperQuota;
        private int TotalQuota;
        private const int qPercentage = 100;
        private bool isConnecting = false;

        public DashBoard()
        {
            InitializeComponent();
        }

        /// <summary>
        /// This method set certain parameters on load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void DashBoard_Load(object sender, EventArgs e)
        {
            startBtn.Enabled = false;
            stopBtn.Enabled = false;
        }

        /// <summary>
        /// This method checks for a connection string then load database data
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void connectBtn_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(DashBoardConnectionString))
            {
                profileQuota();
                ProductionBarIni();

                if (isConnecting)
                {
                    ConnectionPic.Image = Properties.Resources.ok;
                    startBtn.Enabled = true;
                    connectBtn.Enabled = false;
                }
            }
        }

        /// <summary>
        /// This method starts the dashboard
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void startBtn_Click(object sender, EventArgs e)
        {
            refreahRate.Start();
            startBtn.Enabled = false;
            stopBtn.Enabled = true;
            
        }

        /// <summary>
        /// This method stops the station
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void stopBtn_Click(object sender, EventArgs e)
        {
            refreahRate.Stop();
            startBtn.Enabled = true;
            stopBtn.Enabled = false;
        }

        /// <summary>
        /// This method executes store procedures and return a data table
        /// </summary>
        /// <param name="commandString"></param>
        /// <returns></returns>
        private DataTable DashCommands(string commandString)
        {
            OleDbConnection con = new OleDbConnection(DashBoardConnectionString);
            con.Open();
            OleDbCommand cmd = con.CreateCommand();
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = commandString;
            cmd.ExecuteNonQuery();
            DataTable dt = new DataTable();
            OleDbDataAdapter da = new OleDbDataAdapter(cmd);
            da.Fill(dt);
            con.Close();
            return dt;
        }

        /// <summary>
        /// This method gets and displays the production summery
        /// </summary>
        private void productionReport()
        {
            try
            {
                runningPic.Image = Properties.Resources.waiting;
                pSummeryPic.Image = Properties.Resources.waiting;
                ProductionReport.DataSource = DashCommands(DBC.GetPassFail());
                ProductionReport.Series[0].XValueMember = "TotalLamp";
                ProductionReport.Series[0].XValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Int32;
                ProductionReport.Series[0].LegendText = "#VALX (#PERCENT{P2})";
                ProductionReport.Series[0].YValueMembers = "pass_result";
                ProductionReport.Series[0].YValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Int32;
                runningPic.Image = Properties.Resources.ok;
                pSummeryPic.Image = Properties.Resources.ok;
            }
            catch (Exception ex)
            {
                runningPic.Image = Properties.Resources.error;
            }
        }

        /// <summary>
        /// This method gets and display the work stations summary
        /// </summary>
        private void stationSummery()
        {
            try
            {
                sSummaryPic.Image = Properties.Resources.waiting;
                StationSummary.DataSource = DashCommands(DBC.GetStations());
                StationSummary.Series[0].XValueMember = "station_id";
                StationSummary.Series[0].XValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Int32;
                StationSummary.Series[0].YValueMembers = "TotalLamp";
                StationSummary.Series[0].YValueType = System.Windows.Forms.DataVisualization.Charting.ChartValueType.Int32;
                sSummaryPic.Image = Properties.Resources.ok;
            }
            catch (Exception ex)
            {
                sSummaryPic.Image = Properties.Resources.error;
            }
        }

        /// <summary>
        /// This method gets and display the pass fail for each profile
        /// </summary>
        private void productionStatus()
        {
            OleDbConnection con = new OleDbConnection(DashBoardConnectionString);
            con.Open();
            OleDbCommand cmd = con.CreateCommand();
            cmd.CommandType = CommandType.StoredProcedure;
            cmd.CommandText = "sp_DashByProfile";
            cmd.ExecuteNonQuery();
            DataTable dt = new DataTable();
            OleDbDataAdapter da = new OleDbDataAdapter(cmd);
            da.Fill(dt);
            con.Close();

            foreach (DataRow row in dt.Rows)
            {
                switch (Convert.ToInt32(row["profile_id"]))
                {
                    case 1:
                        status = Convert.ToBoolean(row["pass_result"]);
                        if (status)
                        {
                            JouniorPass = Convert.ToInt32(row["TotalLamp"]);
                            p1Pass.Text = Convert.ToString(row["TotalLamp"]);
                        }
                        if (!status)
                        {
                            JouniorFail = Convert.ToInt32(row["TotalLamp"]);
                            p1Fail.Text = Convert.ToString(row["TotalLamp"]);
                        }

                        break;

                    case 2:

                        status = Convert.ToBoolean(row["pass_result"]);
                        if (status)
                        {
                            SeniorPass = Convert.ToInt32(row["TotalLamp"]);
                            p2Pass.Text = Convert.ToString(row["TotalLamp"]);
                        }
                        if (!status)
                        {
                            SeniorFail = Convert.ToInt32(row["TotalLamp"]);
                            p2Fail.Text = Convert.ToString(row["TotalLamp"]);
                        }

                        break;

                    case 3:

                        status = Convert.ToBoolean(row["pass_result"]);
                        if (status)
                        {
                            SuperPass = Convert.ToInt32(row["TotalLamp"]);
                            p3Pass.Text = Convert.ToString(row["TotalLamp"]);
                        }
                        if (!status)
                        {
                            SuperFail = Convert.ToInt32(row["TotalLamp"]);
                            p3Fail.Text = Convert.ToString(row["TotalLamp"]);
                        }

                        break;
                }
            }
        }

        /// <summary>
        /// This method gets the daily quota for each profile
        /// </summary>
        private void profileQuota()
        {
            try
            {
                ConnectionPic.Image = Properties.Resources.waiting;

                OleDbConnection con = new OleDbConnection(DashBoardConnectionString);
                con.Open();
                OleDbCommand cmd = con.CreateCommand();
                cmd.CommandType = CommandType.Text;
                cmd.CommandText = "select profile_id, daily_amount from Profile";
                cmd.ExecuteNonQuery();
                DataTable dt = new DataTable();
                OleDbDataAdapter da = new OleDbDataAdapter(cmd);
                da.Fill(dt);
                con.Close();

                foreach (DataRow row in dt.Rows)
                {
                    switch (Convert.ToInt32(row["profile_id"]))
                    {
                        case 1:
                            JouniorMax = Convert.ToInt32(row["daily_amount"]);

                            break;

                        case 2:

                            SeniorMax = Convert.ToInt32(row["daily_amount"]);

                            break;

                        case 3:

                            SuperMax = Convert.ToInt32(row["daily_amount"]);

                            break;
                    }
                }

                isConnecting = true;
            }
            catch
            {
                isConnecting = false;
                MessageBox.Show("Unable to retrieve station data.\nPlease check database connection config file\nThe incorrect database may have been chosen.", "Database Read Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.Close();
                ConnectionPic.Image = Properties.Resources.error;
            }
        }

        /// <summary>
        /// this method initialize the default bar values
        /// </summary>
        private void ProductionBarIni()
        {
            if (isConnecting)
            {
                p1PassBar.Maximum = JouniorMax;
                p1FailBar.Maximum = JouniorMax;
                p2PassBar.Maximum = SeniorMax;
                p2FailBar.Maximum = SeniorMax;
                p3PassBar.Maximum = SuperMax;
                p3FailBar.Maximum = SuperMax;
                qProgress.Maximum = qPercentage;
            }
        }

        /// <summary>
        /// This method displays the pass and fail for all profiles
        /// </summary>
        private void ProductionBar()
        {
            try
            {
                pStatus.Image = Properties.Resources.waiting;

                if (JouniorPass <= JouniorMax)
                {
                    p1PassBar.Value = JouniorPass;
                    p1FailBar.Value = JouniorFail;
                }

                if (SeniorPass <= SeniorMax)
                {
                    p2PassBar.Value = SeniorPass;
                    p2FailBar.Value = SeniorFail;
                }

                if (SuperPass <= SuperMax)
                {
                    p3PassBar.Value = SuperPass;
                    p3FailBar.Value = SuperFail;
                }

                pStatus.Image = Properties.Resources.ok;
            }
            catch
            {
                pStatus.Image = Properties.Resources.error;
            }
        }

        /// <summary>
        /// This method displays the quota percentage for all profiles.
        /// </summary>
        private void QuotaAmount()
        {
            try
            {
                pQuota.Image = Properties.Resources.waiting;

                int totalJunior = JouniorPass + JouniorFail;
                int totalSenior = SeniorPass + SeniorFail;
                int totalSuper = SuperPass + SuperFail;
                int totalProduction = totalJunior + totalSenior + totalSuper;

                JouniorQuota = (int)Math.Round(((double)(JouniorPass + JouniorFail) / JouniorMax) * qPercentage);
                SeniorQuota = (int)Math.Round(((double)(SeniorPass + SeniorFail) / SeniorMax) * qPercentage);
                SuperQuota = (int)Math.Round(((double)(SuperPass + SuperFail) / SuperMax) * qPercentage);
                TotalQuota = (int)Math.Round(((double)(SuperPass + SeniorPass + JouniorPass) / (SuperMax + SeniorMax + JouniorMax)) * qPercentage);

                if(TotalQuota <= qPercentage) { qProgress.Value = TotalQuota; }
                if (JouniorQuota <= qPercentage) { p1QProgress.Value = JouniorQuota; }
               if (SeniorQuota <= qPercentage) { p2QProgress.Value = SeniorQuota; }
                if (SuperQuota <= qPercentage) { p3QProgress.Value = SuperQuota; }

                T0.Text = totalProduction.ToString() + "/" + (SuperMax + SeniorMax + JouniorMax);
                T1.Text = totalJunior.ToString() + "/" + JouniorMax.ToString();
                T2.Text = totalSenior.ToString() + "/" + SeniorMax.ToString();
                T3.Text = totalSuper.ToString() + "/" + SuperMax.ToString();


                PQ0.Text = TotalQuota.ToString() + " %";
                PQ1.Text = JouniorQuota.ToString() + " %";
                PQ2.Text = SeniorQuota.ToString() + " %";
                PQ3.Text = SuperQuota.ToString() + " %";

                pQuota.Image = Properties.Resources.ok;
            }
            catch
            {
                pQuota.Image = Properties.Resources.error;
            }
        }

        /// <summary>
        /// This method request the data from the database per interval
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>

        private void refreahRate_Tick(object sender, EventArgs e)
        {
            productionReport();
            stationSummery();
            DashBoardDataStatus();
            productionStatus();

            ProductionBar();
            QuotaAmount();
        }

        /// <summary>
        /// This method adjust the bar color based on percentage
        /// </summary>
        private void DashBoardDataStatus()
        {
            qResult = (int)Math.Round(((double)JouniorPass / JouniorMax) * qPercentage);

            p1PassPercent.Text = qResult.ToString() + " %";

            if (qResult > qHigh)
            {
                p1PassBar.ForeColor = Color.Green;
            }
            if (qResult < qHigh)
            {
                p1PassBar.ForeColor = Color.YellowGreen;
            }
            if (qResult < qMid)
            {
                p1PassBar.ForeColor = Color.DarkOrange;
            }
            if (qResult < qLow)
            {
                p1PassBar.ForeColor = Color.Red;
            }

            qResult = (int)Math.Round(((double)JouniorFail / JouniorMax) * qPercentage);

            p1FailPercent.Text = qResult.ToString() + " %";

            if (qResult > qHigh)
            {
                p1FailBar.ForeColor = Color.Green;
            }
            if (qResult < qHigh)
            {
                p1FailBar.ForeColor = Color.YellowGreen;
            }
            if (qResult < qMid)
            {
                p1FailBar.ForeColor = Color.DarkOrange;
            }
            if (qResult < qLow)
            {
                p1FailBar.ForeColor = Color.Red;
            }

            qResult = (int)Math.Round(((double)SeniorPass / SeniorMax) * qPercentage);

            p2PassPercent.Text = qResult.ToString() + " %";

            if (qResult > qHigh)
            {
                p2PassBar.ForeColor = Color.Green;
            }
            if (qResult < qHigh)
            {
                p2PassBar.ForeColor = Color.YellowGreen;
            }
            if (qResult < qMid)
            {
                p2PassBar.ForeColor = Color.DarkOrange;
            }
            if (qResult < qLow)
            {
                p2PassBar.ForeColor = Color.Red;
            }

            qResult = (int)Math.Round(((double)SeniorFail / SeniorMax) * qPercentage);

            p2FailPercent.Text = qResult.ToString() + " %";

            if (qResult > qHigh)
            {
                p2FailBar.ForeColor = Color.Green;
            }
            if (qResult < qHigh)
            {
                p2FailBar.ForeColor = Color.YellowGreen;
            }
            if (qResult < qMid)
            {
                p2FailBar.ForeColor = Color.DarkOrange;
            }
            if (qResult < qLow)
            {
                p2FailBar.ForeColor = Color.Red;
            }

            qResult = (int)Math.Round(((double)SuperPass / SuperMax) * qPercentage);

            p3PassPercent.Text = qResult.ToString() + " %";

            if (qResult > qHigh)
            {
                p3PassBar.ForeColor = Color.Green;
            }
            if (qResult < qHigh)
            {
                p3PassBar.ForeColor = Color.YellowGreen;
            }
            if (qResult < qMid)
            {
                p3PassBar.ForeColor = Color.DarkOrange;
            }
            if (qResult < qLow)
            {
                p3PassBar.ForeColor = Color.Red;
            }

            qResult = (int)Math.Round(((double)SuperFail / SuperMax) * qPercentage);

            p3FailPercent.Text = qResult.ToString() + " %";

            if (qResult > qHigh)
            {
                p3FailBar.ForeColor = Color.Green;
            }
            if (qResult < qHigh)
            {
                p3FailBar.ForeColor = Color.YellowGreen;
            }
            if (qResult < qMid)
            {
                p3FailBar.ForeColor = Color.DarkOrange;
            }
            if (qResult < qLow)
            {
                p3FailBar.ForeColor = Color.Red;
            }

            qResult = (int)Math.Round(((double)(SuperPass + SeniorPass + JouniorPass) / (SuperMax + SeniorMax + JouniorMax)) * qPercentage);

            if (qResult > qHigh)
            {
                qProgress.ForeColor = Color.Green;
            }
            if (qResult < qHigh)
            {
                qProgress.ForeColor = Color.YellowGreen;
            }
            if (qResult < qMid)
            {
                qProgress.ForeColor = Color.DarkOrange;
            }
            if (qResult < qLow)
            {
                qProgress.ForeColor = Color.Red;
            }

            qResult = (int)Math.Round(((double)SeniorPass / SeniorMax) * qPercentage);

            if (qResult > qHigh)
            {
                p2QProgress.ForeColor = Color.Green;
            }
            if (qResult < qHigh)
            {
                p2QProgress.ForeColor = Color.YellowGreen;
            }
            if (qResult < qMid)
            {
                p2QProgress.ForeColor = Color.DarkOrange;
            }
            if (qResult < qLow)
            {
                p2QProgress.ForeColor = Color.Red;
            }

            qResult = (int)Math.Round(((double)SuperPass / SuperMax) * qPercentage);

            if (qResult > qHigh)
            {
                p3QProgress.ForeColor = Color.Green;
            }
            if (qResult < qHigh)
            {
                p3QProgress.ForeColor = Color.YellowGreen;
            }
            if (qResult < qMid)
            {
                p3QProgress.ForeColor = Color.DarkOrange;
            }
            if (qResult < qLow)
            {
                p3QProgress.ForeColor = Color.Red;
            }

            qResult = (int)Math.Round(((double)JouniorPass / JouniorMax) * qPercentage);

            if (qResult > qHigh)
            {
                p1QProgress.ForeColor = Color.Green;
            }
            if (qResult < qHigh)
            {
                p1QProgress.ForeColor = Color.YellowGreen;
            }
            if (qResult < qMid)
            {
                p1QProgress.ForeColor = Color.DarkOrange;
            }
            if (qResult < qLow)
            {
                p1QProgress.ForeColor = Color.Red;
            }
        }

        /// <summary>
        /// This method determine the refresh interval base on user define
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void refreshInterval_Scroll(object sender, EventArgs e)
        {
            refreahRate.Interval = Convert.ToInt32(refreshInterval.Value);
        }
    }
}