-- Store Procedure - Advanced SQL - Final project
-- Author:	Laura Fadrique
-- Date:	April 08, 2017
-- STORE PROCEDURE AssemblyLamp

USE Kanban;
GO

IF (OBJECT_ID('sp_AssemblyLamp') IS NOT NULL)
  DROP PROCEDURE sp_AssemblyLamp
GO

CREATE PROCEDURE sp_AssemblyLamp
	@paramStationId INT,
	@paramDateProduction DATE = null,
	@paramDefect INT
AS
IF @paramDateProduction IS null
    SET @paramDateProduction=getdate()
BEGIN
	DECLARE	
	@LampId INT,
	@TrayID INT = 0,
	@QtyTray INT = 0,
	@BinID INT,
	@MaxTray INT

	SELECT @MaxTray = tray_capacity FROM config

	-- create cursos to decrease all the bins
	DECLARE binCursor CURSOR FOR   
		SELECT bin_ID FROM Bin 
		WHERE station_id = @paramStationId 
		AND date_production = @paramDateProduction
		AND current_qty > 0
	
	OPEN binCursor

	FETCH NEXT FROM binCursor INTO @BinID

	BEGIN TRAN
		WHILE (@@FETCH_STATUS=0)
		BEGIN
		-- Get current bin number for the station
			UPDATE Bin SET current_qty = current_qty - 1
			WHERE bin_ID = @BinID

			IF @@ERROR <>0
			BEGIN
				ROLLBACK TRAN
			END

			FETCH NEXT FROM binCursor INTO @BinID
		END

		-- select tray to be increased
		SELECT @TrayID = tray_ID, @QtyTray = current_qty FROM Tray
		WHERE station_id = @paramStationId
		AND current_qty <> @MaxTray
		ORDER BY tray_ID DESC
		-- if no tray to increase get a new one
		IF (@TrayID = 0)
			-- assign new tray
			EXEC dbo.sp_AssignTray @paramStationId, @TrayID output

		--Increase tray
		-- Get current bin number for the station
		UPDATE Tray SET current_qty = current_qty + 1
		WHERE tray_ID = @TrayID
		IF @@ERROR <> 0
		BEGIN
			ROLLBACK TRAN
		END
		-- Set the test result

		-- Save production/assembly info
		INSERT INTO production 
		VALUES (@paramStationId, @paramDateProduction, @QtyTray + 1, @TrayID, @paramDefect)
		IF @@ERROR <> 0
		BEGIN
			ROLLBACK TRAN
		END
	COMMIT TRAN
	CLOSE binCursor
	DEALLOCATE binCursor
END

GO





