-- Store Procedure - Advanced SQL - Final project
-- Author:	Laura Fadrique
-- Date:	April 15, 2017
-- STORE PROCEDURE CheckBin

USE Kanban;
GO

IF (OBJECT_ID('sp_CheckBin') IS NOT NULL)
  DROP PROCEDURE sp_CheckBin
GO

CREATE PROCEDURE sp_CheckBin
	@paramStationId INT,
	@paramDateProduction DATE = null
AS
IF @paramDateProduction IS null
    SET @paramDateProduction=getdate()
BEGIN
	DECLARE	
	@minPart INT,
	@BinID INT,
	@PartID INT,
	@currentQty INT

	-- create cursos to traverse current bins
	DECLARE binCursor CURSOR FOR   
		SELECT bin_ID, part_ID, current_qty FROM Bin 
		WHERE station_id = @paramStationId 
		AND date_production = @paramDateProduction
		AND current_qty > 0
	
	OPEN binCursor

	FETCH NEXT FROM binCursor INTO @BinID, @PartID, @currentQty

	WHILE (@@FETCH_STATUS=0)
	BEGIN
		-- Get the mininum capacity for a specific partId
		SELECT @minPart = min_bin_capacity FROM Part
				WHERE part_Id = @PartID

		IF (@minPart >= @currentQty)
			BEGIN 
				BEGIN TRAN
					UPDATE Bin
					SET runner_flag = 1
					WHERE bin_id = @BinID

					IF @@ERROR <> 0
					BEGIN
						ROLLBACK TRAN
					END
				COMMIT TRAN
			END

		FETCH NEXT FROM binCursor INTO @BinID, @PartID, @currentQty
	END

	CLOSE binCursor
	DEALLOCATE binCursor

END
GO