-- Store Procedure - Advanced SQL - Final project
-- Author:	Laura Fadrique
-- Date:	April 15, 2017
-- STORE PROCEDURE ReloadBin

USE Kanban;
GO

IF (OBJECT_ID('sp_ReloadBin') IS NOT NULL)
  DROP PROCEDURE sp_ReloadBin
GO

CREATE PROCEDURE sp_ReloadBin
	@paramBinId INT,
	@paramStationId INT = null,
	@paramPartId INT
AS
BEGIN
	DECLARE	
	@paramDateProduction DATE = getdate(),
	@nextBinId INT,
	@currentQty INT,
	@maxQty INT

	SELECT @maxQty = max_bin_capacity, @currentQty = b.current_qty FROM Part p
	INNER JOIN Bin b ON p.part_id = b.part_id
	AND b.bin_id = @paramBinId

	SELECT TOP 1 @nextBinId = (bin_id) FROM Bin
	WHERE part_id = @paramPartId
	AND station_id IS NULL

	BEGIN TRAN

		UPDATE Bin
		SET current_qty = @maxQty + @currentQty,
			station_id = @paramStationId,
			date_production = @paramDateProduction
		WHERE bin_id = @nextBinId
		IF @@ERROR <> 0
		BEGIN
			ROLLBACK TRAN
		END

		UPDATE Bin
		SET runner_flag = 0,
			current_qty = 0
		WHERE bin_id = @paramBinId
		IF @@ERROR <> 0
		BEGIN
			ROLLBACK TRAN
		END

	COMMIT TRAN
END

GO

