﻿/*
File name:			DB.cs
Project name:		SQL - Final Project
Developer:			Laura Fadrique - ID 7380801 / Brian Hinds - ID 6873343
Date:				March 31, 2017
Description:		DB class.
                    Handles all the methos related to the DB.
                    Connection, selects, etc.
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;

namespace SQLConfig
{
    class DB
    {
        private SqlConnection conn = new SqlConnection();                               //Connection with the Database
        private SqlTransaction trans;                                                   //Database Transaction 
        private SqlCommand cmd = new SqlCommand();                                      //Database Command
        private bool Connected = false;                                                 //Tells if the Database is connected or not



        /* -- METHOD COMMENT --
        NAME	:	IsConnected
        INPUT	:	Nothing
        OUTPUT	:	Nothing
        PURPOSE :	Return if the Database is Connected.
        */
        public bool isConnected
        {
            get
            {
                return Connected;
            }
        }





        /* -- METHOD COMMENT --
        NAME	:	Connect
        INPUT	:	Nothing
        OUTPUT	:	Nothing
        PURPOSE :	Connect with a database using the Data Link Properties.
			        Initialize the data types for SQL and Access
        */
        public void Connect()
        {
            try
            {
                string connection = ConfigurationManager.ConnectionStrings["SQLConfig.Properties.Settings.KanbanConnectionString"].ConnectionString;

                conn.ConnectionString = connection;
                Connected = true;


            }
            catch (Exception e)
            {
                Connected = false;
            }

        }



        /* -- METHOD COMMENT --
        NAME	:	Disconnect
        INPUT	:	Nothing
        OUTPUT	:	Nothing
        PURPOSE :	Disconnect the current database.
        */
        public void Disconnect()
        {
            try
            {
                cmd.Dispose();
                conn.Dispose();
                Connected = false;

            }
            catch (Exception e)
            {
                Connected = false;
            }

        }



        /* -- METHOD COMMENT --
        NAME	:	SelectTable
        INPUT	:	string table        --> table to be selected
                    string fields       --> the fields for the select
                    string order        --> order clause
                    string where        --> where clause
        OUTPUT	:	DataTable           --> all the data from the table
        PURPOSE :	Return all the data of a specific table according to the arguments.
        */
        public DataTable SelectTable(string table, string fields, string order, string where)
        {
            try
            {
                //Query to select all the data
                string query = "SELECT " + fields + " FROM " + table;

                if (where != "ALL")
                {
                    query += " WHERE " + where;
                }

                if (order != "ALL")
                {
                    query += " ORDER BY " + order;
                }

                //Open connection
                conn.Open();

                //Start command
                cmd.Connection = conn;
                cmd.CommandText = query;

                //Create data adapter
                SqlDataAdapter da = new SqlDataAdapter(query, conn);
                DataTable dt = new DataTable();
                da.Fill(dt);

                conn.Close();
                return dt;

            }
            catch (Exception ex)
            {
                conn.Close();
                throw ex;
            }

        }

        /* -- METHOD COMMENT --
        NAME	:	InsertTable
        INPUT	:	string table            --> The table name to be inserted
                    string values            --> values to be inserted
        OUTPUT	:	Int                     --> returns the number of rows affected
        PURPOSE :	Insert the content of a string in a table existing in the database.
        */
        public int InsertRow(string table, string values)
        {
            //Open database
            conn.Open();
            //Begin the transaction
            trans = conn.BeginTransaction();
            cmd.Transaction = trans;
            int result = 0;
            try
            {
                //Assign the query to the command
                cmd.CommandText = "INSERT INTO " + table
                    + " VALUES (" + values + ");";
                //Execute the Non Query
                result = cmd.ExecuteNonQuery();

                //Commit the transaction
                trans.Commit();
                conn.Close();
                return result;
            }
            catch (Exception ex)
            {
                //Rollback in case of error
                trans.Rollback();
                conn.Close();
                throw ex;
            }

        }


    }
}
