﻿/*
File name:			Form1.cs
Project name:		SQL - Final Project
Developer:			Laura Fadrique - ID 7380801 / Brian Hinds - ID 6873343
Date:				March 31, 2017
Description:		Form Class.
                    Kanban project.
                    Config form. 
                    Maintain information for the Config table.
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SQLConfig
{
    public partial class frmConfig : Form
    {
        DB myDB = new DB();
        bool configChange = false;

        public frmConfig()
        {
            InitializeComponent();
            tabPage1.Text = @"General";
            tabPage2.Text = @"Product";
            tabPage3.Text = @"Profile";
        }

        /* -- METHOD COMMENT --
        NAME	:	btnCancel_Click
        PURPOSE :	Close the form and disconnect the DB.
        */
        private void btnCancel_Click(object sender, EventArgs e)
        {
            myDB.Disconnect();
            this.Close();
        }

        /* -- METHOD COMMENT --
        NAME	:	frmConfig_Load
        PURPOSE :	Load th data grid for Profile and Parts.
                    Load the values for General Config.
                    Connect to the database.
        */
        private void frmConfig_Load(object sender, EventArgs e)
        {
            // TODO: This line of code loads data into the 'kanbanDataSet.Profile' table. You can move, or remove it, as needed.
            this.profileTableAdapter.Fill(this.kanbanDataSet.Profile);
            // TODO: This line of code loads data into the 'kanbanDataSet1.Part' table. You can move, or remove it, as needed.
            this.partTableAdapter.Fill(this.kanbanDataSet1.Part);
            //Connect to database
            myDB.Connect();
            //load the first tab
            LoadGenaralTab();

        }


        /* -- METHOD COMMENT --
        NAME	:	LoadGenaralTab
        INPUT	:	Nothing
        OUTPUT	:	Nothing
        PURPOSE :	Load the values of the general Config.
			        Get the last config date.
        */
        private void LoadGenaralTab()
        {
            try
            {

                DataTable dt = new DataTable();

                //parameters for the select
                string fields = "TOP 1 [date_config] ,[runner_time],[max_station],[tray_capacity],[avg_assembly],[scale_flag] ";
                string table = "[Kanban].[dbo].[Config]";
                string order = "[date_config] DESC";

                //Call DB method
                dt = myDB.SelectTable(table, fields, order, "ALL");

                if (dt.Rows.Count > 0)
                {
                    //Assign values
                    txtDate.Text = dt.Rows[0]["date_config"].ToString().Substring(0, 10);
                    nudRunnerTime.Value = Convert.ToDecimal(dt.Rows[0]["runner_time"]);
                    nudMaxStation.Value = Convert.ToInt32(dt.Rows[0]["max_station"]);
                    nudTrayCapacity.Value = Convert.ToInt32(dt.Rows[0]["tray_capacity"]);
                    nudAvgTime.Value = Convert.ToInt32(dt.Rows[0]["avg_assembly"]);
                    chkScaleFlag.Checked = Convert.ToBoolean(dt.Rows[0]["scale_flag"]);
                }
                configChange = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Warning!");
            }
        }



        /* -- METHOD COMMENT --
        NAME	:	btnNext2_Click
        PURPOSE :	Save changes in the parts tab and opens next tab (profile).
        */
        private void btnNext2_Click(object sender, EventArgs e)
        {
            //Ask if want to save changes.
            DialogResult result = MessageBox.Show("Do you want to save the changes in Parts?", "Important",MessageBoxButtons.YesNo );
            if (result == DialogResult.Yes)
            {
                // Save changes in the Parts Tab
                dgvParts.EndEdit();
                partTableAdapter.Update(kanbanDataSet1);
            }
            //go to the next tab - profile
            tabConfig.SelectedIndex = tabConfig.TabCount - 1;  //Last tab
        }



        /* -- METHOD COMMENT --
        NAME	:	btnNext1_Click
        PURPOSE :	Save changes in the general config tab and opens next tab (parts).
        */
        private void btnNext1_Click(object sender, EventArgs e)
        {
            //Save changes for the general config
            SaveGeneralConfig();
            tabConfig.SelectedIndex = tabConfig.SelectedIndex + 1;  //Last tab
        }



        /* -- METHOD COMMENT --
        NAME	:	SaveGeneralConfig
        INPUT	:	Nothing
        OUTPUT	:	Nothing
        PURPOSE :	Save the general config to the database.
        */
        private void SaveGeneralConfig()
        {
            try
            {
                if (configChange)
                {
                    DialogResult result = MessageBox.Show("Do you want to save the changes in General Config?", "Important", MessageBoxButtons.YesNo);
                    if (result == DialogResult.Yes)
                    {
                        string values = "'" + DateTime.Now.ToString() + "'";

                        values += ", " + nudRunnerTime.Value.ToString();
                        values += ", " + nudMaxStation.Value.ToString();
                        values += ", " + nudTrayCapacity.Value.ToString();
                        values += ", " + nudAvgTime.Value.ToString();
                        if (chkScaleFlag.Checked)
                        {
                            values += ", " + 1;
                        }
                        else
                        {
                            values += ", " + 0;
                        }

                        int resultRows = myDB.InsertRow("Config", values);
                    }
                }

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Warning!");
            }
        }


        /* -- METHOD COMMENT --
        NAME	:	btnSave_Click
        PURPOSE :	Save changes in the profile tab.
        */
        private void btnSave_Click(object sender, EventArgs e)
        {
            try
            {
                DialogResult result = MessageBox.Show("Do you want to save the changes in Profile?", "Important", MessageBoxButtons.YesNo);
                if (result == DialogResult.Yes)
                {
                    //Save changes in the Profile Tab
                    dgvProfile.EndEdit();
                    profileTableAdapter.Update(kanbanDataSet);
                }

                MessageBox.Show("Config values saved!", "Info");
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message.ToString(), "Warning!");
            }
        }



        private void fillToolStripButton_Click(object sender, EventArgs e)
        {
            try
            {
                this.partTableAdapter.Fill(this.kanbanDataSet1.Part);
            }
            catch (System.Exception ex)
            {
                System.Windows.Forms.MessageBox.Show(ex.Message);
            }

        }


        /* -- METHOD COMMENT --
        NAME	:	frmConfig_FormClosing
        PURPOSE :	Disconnect the db when close the form
        */
        private void frmConfig_FormClosing(object sender, FormClosingEventArgs e)
        {
            //Disconnect the DB;
            myDB.Disconnect();
        }



        /* -- METHOD COMMENT --
        NAME	:	nudRunnerTime_ValueChanged
        PURPOSE :	Verify if the data was changed for General Config
        */
        private void nudRunnerTime_ValueChanged(object sender, EventArgs e)
        {
            configChange = true;
        }



        /* -- METHOD COMMENT --
        NAME	:	nudMaxStation_ValueChanged
        PURPOSE :	Verify if the data was changed for General Config
        */
        private void nudMaxStation_ValueChanged(object sender, EventArgs e)
        {
            configChange = true;
        }



        /* -- METHOD COMMENT --
        NAME	:	nupTrayCapacity_ValueChanged
        PURPOSE :	Verify if the data was changed for General Config
        */
        private void nupTrayCapacity_ValueChanged(object sender, EventArgs e)
        {
            configChange = true;
        }



        /* -- METHOD COMMENT --
        NAME	:	nudAvgTime_ValueChanged
        PURPOSE :	Verify if the data was changed for General Config
        */
        private void nudAvgTime_ValueChanged(object sender, EventArgs e)
        {
            configChange = true;
        }


        /* -- METHOD COMMENT --
        NAME	:	chkScaleFlag_CheckedChanged
        PURPOSE :	Verify if the data was changed for General Config
        */
        private void chkScaleFlag_CheckedChanged(object sender, EventArgs e)
        {
            configChange = true;
        }
    }
}
