-- Store Procedure - Advanced SQL - Final project
-- Author:	Laura Fadrique
-- Date:	March 05, 2017
-- STORE TRIGGER tr_AssignBinStation

USE Kanban;
GO

IF (OBJECT_ID('tr_AssignBinStation') IS NOT NULL)
  DROP TRIGGER tr_AssignBinStation
GO


CREATE TRIGGER tr_AssignBinStation
ON Station AFTER INSERT
AS
BEGIN
	-- Assign 6 first bins for the station
	DECLARE 
	@ProductId INT,
	@StationId INT,
	@DateProduction DATE,
	@PartID INT,
	@MaxQty INT,
	@BinID INT

	SELECT @ProductId = INSERTED.product_id FROM INSERTED
	SELECT @StationId = INSERTED.station_id FROM INSERTED
	SELECT @DateProduction = INSERTED.date_production FROM INSERTED

	-- create cursos to get parts info
	DECLARE partCursor CURSOR FOR   
		SELECT p.part_ID, p.max_bin_capacity 
		FROM Part p, product_part pp 
		WHERE p.part_ID = pp.part_ID 
		AND pp.product_id = 1
	
	OPEN partCursor

	FETCH NEXT FROM partCursor INTO @PartID, @MaxQty

	WHILE (@@FETCH_STATUS=0)
	BEGIN
		BEGIN TRAN
			SELECT TOP 1 @BinID = bin_ID FROM Bin
			WHERE part_ID = @PartID
			AND current_qty IS NULL
			AND station_id IS NULL

			-- Update the bin with max_qty, stationID and current_date
			UPDATE Bin SET station_ID = @StationId,
			date_production = @DateProduction,
			current_qty = @MaxQty
			WHERE bin_ID = @BinID

			FETCH NEXT FROM partCursor INTO @PartID, @MaxQty
		COMMIT TRAN
	END
	CLOSE partCursor
	DEALLOCATE partCursor
END 